@@ -1,4 +1,4 @@
-""" PhotosAlbum class to create an album in default Photos library and add photos to it """
+"""PhotosAlbum class to create an album in default Photos library and add photos to it"""
 
 from __future__ import annotations
 
@@ -118,11 +118,16 @@ class PhotosAlbum:
         self.library = photoscript.PhotosLibrary()
 
         folders_album = name.split(split_folder) if split_folder else [name]
+        if folders_album and folders_album[0] == "":
+            folders_album = folders_album[1:]
         self.album = album_by_path(folders_album, verbose=verbose)
         self.name = name
         self.rich = rich
 
     def add(self, photo: PhotoInfo):
+        if photo.shared or (photo.syndicated and not photo.saved_to_library):
+            # shared / unsaved syndicated photos cannot be added to albums
+            return
         photo_ = photoscript.Photo(photo.uuid)
         self.album.add([photo_])
         self.verbose(
@@ -132,6 +137,9 @@ class PhotosAlbum:
     def update(self, photos: Iterable[PhotoInfo]):
         photoscript_photos = []
         for p in photos:
+            if p.shared or (p.syndicated and not p.saved_to_library):
+                # shared / unsaved syndicated photos cannot be added to albums
+                continue
             try:
                 photoscript_photos.append(photoscript.Photo(p.uuid))
             except Exception as e:
@@ -198,6 +206,7 @@ class PhotosAlbumPhotoScript(PhotosAlbum):
         """Add list of photos to album"""
         self.update(photos)
 
+
 class PhotosAlbumPhotoScriptByPath(PhotosAlbumPhotoScript):
     """Add photoscript.Photo objects to album"""
 
